/*
   Copyright (C) 1998  Ulric Eriksson <ulric@edu.stockholm.se>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the Licence, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */

/* 
 * Placement.c - Placement composite widget
 */

#include	<X11/IntrinsicP.h>
#include	<X11/StringDefs.h>
#include	<X11/Xmu/Misc.h>
#include	<X11/Xaw/Command.h>
#include	"PlacementP.h"

/****************************************************************
 *
 * Placement Resources
 *
 ****************************************************************/

/* These are bogus; they are hand-me-downs from Box */

#define offset(field) XtOffsetOf(PlacementRec, placement.field)
static XtResource resources[] = {
	{
		XtNplacer,		/* name */
		XtCPlacer,		/* class */
		XtRPointer,		/* type */
		sizeof(XtPointer),	/* size */
		offset(placer),		/* offset */
		XtRImmediate,		/* default_type */
		(XtPointer)NULL		/* default_addr */
	}
};

/****************************************************************
 *
 * Full class record constant
 *
 ****************************************************************/

#if 0
static void Initialize();
#endif
static void Resize();
static Boolean SetValues();
static XtGeometryResult GeometryManager();
static void ChangeManaged();

PlacementClassRec placementClassRec = {
  {
/* core_class fields      */
    /* superclass         */    (WidgetClass) &compositeClassRec,
    /* class_name         */    "Placement",
    /* widget_size        */    sizeof(PlacementRec),
    /* class_initialize   */    NULL,
    /* class_part_init    */	NULL,
    /* class_inited       */	FALSE,
    /* initialize         */    NULL,
    /* initialize_hook    */	NULL,
    /* realize            */    XtInheritRealize,
    /* actions            */    NULL,
    /* num_actions	  */	0,
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion	  */	TRUE,
    /* compress_exposure  */	TRUE,
    /* compress_enterleave*/	TRUE,
    /* visible_interest   */    FALSE,
    /* destroy            */    NULL,
    /* resize             */    Resize,
    /* expose             */    NULL,
    /* set_values         */    SetValues,
    /* set_values_hook    */	NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,
    /* get_values_hook    */	NULL,
    /* accept_focus       */    NULL,
    /* version            */	XtVersion,
    /* callback_private   */    NULL,
    /* tm_table           */    NULL,
    /* query_geometry     */	XtInheritQueryGeometry,
    /* display_accelerator*/	XtInheritDisplayAccelerator,
    /* extension          */	NULL
  },{
/* composite_class fields */
    /* geometry_manager   */    GeometryManager,
    /* change_managed     */    ChangeManaged,
    /* insert_child	  */	XtInheritInsertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension          */	NULL
  },{
/* Placement class fields */
    /* empty		  */	0
  }
};

WidgetClass placementWidgetClass = (WidgetClass)&placementClassRec;


/****************************************************************
 *
 * Private Routines
 *
 ****************************************************************/

/*
 * Do a layout, actually assigning positions.
 */

static void DoLayout(PlacementWidget sw)
{
	int i;
	PlacerStruct *ps;

	if (!sw->placement.placer) return;	/* no clue */
	ps = (*sw->placement.placer)(sw->core.width,
					sw->core.height,
					sw->composite.children,
					sw->composite.num_children);
	for (i = 0; i < sw->composite.num_children; i++) {
		XtConfigureWidget(sw->composite.children[i],
				ps[i].x, ps[i].y, ps[i].w, ps[i].h, ps[i].b);
	}
}

/*
 * Actually layout placement
 */

static void Resize(Widget w)
{
	DoLayout((PlacementWidget)w);
} /* Resize */

/*
 * Geometry Manager
 *
 * 'reply' is unused; we say only yeay or nay, never almost.
 */

static XtGeometryResult GeometryManager(Widget w,
		XtWidgetGeometry request, XtWidgetGeometry reply)
{
	return XtGeometryYes;
}

static void ChangeManaged(Widget w)
{
    DoLayout((PlacementWidget)w);
}

static Boolean SetValues(Widget current, Widget request, Widget new,
		ArgList args, Cardinal *num_args)
{
    return False;
}
